﻿var ParticleSystem = function() {
	
	this.x = 0;
	this.y = 0;
	
	this.visible = true;
	
	this.isRunning = false;
	this.isPaused = false;
	
	this.particles = [];
	this.count = 0;
	
	this.lifespan = 0;
	this.lifeRandom = 0;
	
	// basic speed
	this.dx = 0;
	this.dy = 0;
	
	// random element
	this.dxR = 0;
	this.dyR = 0;
	
	this.scaling = 1;
	
	// gravity etc
	this.forceX = 0;
	this.forceY = 0;
	
	this.spawnInterval = 1;
	this.spawnNext = 0;
	
	// spawn area
	this.spawnWidth = 0;
	this.spawnHeight = 0;
	
	// frame offset allows us to start a particle system a number
	// of frames in (so we don't have to wait for a stream to get going)
	this.frameOffset = 0;
	
	// for set duration bursts
	this.duration = 0;
	
	this.willComplete = false;
	this.isComplete = false;
}

ParticleSystem.prototype = {
	
	init : function(asset, num, life, dx, dy) {
		
		this.count = num;
		this.lifespan = life;
		
		this.dx = dx;
		this.dy = dy;
		
		// create all particles
		var i = this.count;
		while (i-- > 0) {
			
			// create particle and put in list
			var p = new Particle(asset);
			this.particles.push(p);
		}
	},
	
	start : function(d) {
		
		this.duration = d || 0;
		this.isRunning = true;
		
		if (this.duration > 0) this.willComplete = true;
		else this.willComplete = false;
		
		this.isComplete = false;
		
		// do any frame offsetting
		var i = this.frameOffset;
		
		while (i-- > 0) {
			
			this.update();
		}
	},
	
	stop : function() {
		
		this.isRunning = false;
	},
	
	pause : function() {
		
		this.isPaused = true;
	}, 
	
	resume : function() {
		
		this.isPaused = false;
	},
	
	burst : function() {
		
		if (!this.paused) {
			
			var i = this.count;
			
			while (i-- > 0) {
				
				this.addParticle(this.randomSpeed(this.dx, this.dxR), this.randomSpeed(this.dy, this.dyR), this.randomLife(this.lifespan));
			}
		}
		
		this.willComplete = true;
		this.isComplete = false;
	},
	
	addParticle : function (dx, dy, life) {
		
			// find next free particle
			var i = this.count;
			while (i-- > 0) {
				
				var p = this.particles[i];
				
				// unused particles are hidden
				if (!p.visible) {
					
					// break loop when free particle is found
					break;
				}
			}
			
			// set start position
			p.x = this.x;
			p.y = this.y;
			
			if (this.spawnWidth != 0) {
				
				p.x += (Math.random() * this.spawnWidth) - (this.spawnWidth * 0.5);
			}
			
			if (this.spawnHeight != 0) {
				
				p.y += (Math.random() * this.spawnHeight) - (this.spawnHeight * 0.5);
			}
			
			p.setScale(1);
			
			p.maxLife = life;
			p.life = life;
			p.fade = 1 / life;
			
			p.dx = dx;
			p.dy = dy;
			
			p.scaling = this.scaling;
			
			// show particle
			p.visible = true;
	},
	
	randomSpeed : function(d, r) {
		
		// deviation can be positive or negative
		var r = (((d + 1) * r) * Math.random()) - (((d + 1) * r) * 0.5);
		d += r;
		
		return d;
	},
	
	randomLife : function(life) {
		
		// deviation may ONLY be negative (so we always know the maximum value)
		var r = ((life + 1) * this.lifeRandom) * Math.random();
		life -= r;
		
		return life;
	},
	
	setForces : function(fx, fy) {
		
		this.forceX = fx || 0;
		this.forceY = fy || 0;
	},
	
	setScaling : function(scale) {
		
		this.scaling = scale;
	},
	
	setLifeRandomness : function(random) {
		
		this.lifeRandom = random;
	},
	
	setSpeed : function(x, y) {
		
		this.dx = x;
		this.dy = y;
	},
	
	setSpeedRandomness : function(x, y) {
		
		this.dxR = x;
		this.dyR = y;
	},
	
	setSpawnArea : function(x, y) {
		
		this.spawnWidth = x;
		this.spawnHeight = y;
	},

	setSpawnInterval : function(interval) {
		
		this.spawnInterval = interval;
	},
	
	setFrameOffset : function(offset) {
		
		this.frameOffset = offset;
	},
	
	update : function() {
		
		if (this.isPaused) return;
		
		if (this.duration > 0) {
		
			if (--this.duration == 0) this.stop();
		}
		
		if (this.isRunning && !this.isPaused) {
			
			if (this.spawnNext >= this.spawnInterval) {
				
				// add a particle every frame while the system is running
				this.addParticle(this.randomSpeed(this.dx, this.dxR), this.randomSpeed(this.dy, this.dyR), this.randomLife(this.lifespan));
				this.spawnNext = 0;
			}
			
			this.spawnNext++;
		}
		
		var i = this.count;
		
		while (i-- > 0) {
			
			var p = this.particles[i];
			
			if (p.visible) {
				
				// reduce life
				p.life--;
				
				if (p.life <= 0) {
					
					// remove the particle when dead
					p.visible = false;
				}
				else {
					
					// is there any gravity/forces
					p.dy += this.forceY;
					p.dx += this.forceX;
					
					// update position
					p.x += p.dx;						
					p.y += p.dy;
					
					// update size
					p.setScale(p.scale * p.scaling);
				}
			}
		}
		
		// check for completion
		if (this.willComplete && !this.isComplete) {
			
			var complete = true;
			i = this.count;
			
			while (i-- > 0) {
				
				// all particles hidden
				if (this.particles[i].visible) {
					
					complete = false;
					break;
				}
			}
			
			if (complete) {
				
				//dispatchEvent(new Event("complete"));
				this.isComplete = true;
			}
		}
	},
	
	draw : function () {
		
		var i = this.count;
		
		while (i-- > 0) {
			
			this.particles[i].draw();
		}
	},
	
	cleanUp : function() {
		
		this.particles = null;
	},
	
	reset : function() {
		
		var i = this.count;
		
		while (i-- > 0) {
			
			this.particles[i].visible = false;
		}
	}
}